---------------------------------------------------------------------------------------------
--- Step 1: Upgrade stored procedures
---------------------------------------------------------------------------------------------
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[sp_sc_disable_indexes]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[sp_sc_disable_indexes]
GO
-- =============================================
-- Author:		Dmitry Kostenko
-- Create date: 10-12-2011
-- Description:	Disable non-clustered indexes on a specific table or all tables in the database
-- =============================================
CREATE PROCEDURE [sp_sc_disable_indexes]
  @tablename varchar(100) = null
AS
declare @indextable varchar(100), @indexname varchar(100), @sql nvarchar(1000)
  declare indexes cursor for 
    select OBJECT_NAME(sys.indexes.object_id) indextable, sys.indexes.name indexname
    from 
      sys.indexes 
      inner join sys.tables on sys.indexes.object_id = sys.tables.object_id 
    where 
       sys.indexes.type_desc <> 'CLUSTERED' and (@tablename is null or OBJECT_NAME(sys.indexes.object_id) = @tablename)
BEGIN
  open indexes
  fetch next from indexes into @indextable, @indexname
  while (@@FETCH_STATUS<>-1) begin
    set @sql = 'ALTER INDEX ' + @indexname + ' ON ' + @indextable + ' DISABLE'
    exec sp_executesql @sql
    fetch next from indexes into @indextable, @indexname
  end
  close indexes
  deallocate indexes
END
GO
---------------------------------------------------------------------------------------------
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[sp_sc_enable_indexes]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[sp_sc_enable_indexes]
GO
-- =============================================
-- Author:		Dmitry Kostenko
-- Create date: 10-12-2011
-- Description:	Enable all non-clustered indexes in the database before loading data
-- =============================================
CREATE PROCEDURE [sp_sc_enable_indexes]
	-- Add the parameters for the stored procedure here
	@tablename varchar(100) = null
AS
declare @indextable varchar(100), @indexname varchar(100), @sql nvarchar(1000)
  declare indexes cursor for 
    select OBJECT_NAME(object_id) indextable, name indexname 
    from sys.indexes 
    where is_disabled = 1 and (@tablename is null or OBJECT_NAME(object_id) = @tablename)
    order by type_desc
BEGIN
  open indexes
  fetch next from indexes into @indextable, @indexname
  while (@@FETCH_STATUS<>-1) begin
    set @sql = 'ALTER INDEX [' + @indexname + '] ON [' + @indextable+ '] REBUILD'
    exec sp_executesql @sql
    fetch next from indexes into @indextable, @indexname
  end
  close indexes
  deallocate indexes
END

GO

---------------------------------------------------------------------------------------------
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[sp_sc_disable_constraints]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[sp_sc_disable_constraints]
GO
-- =============================================
-- Author:		Dmitry Kostenko
-- Create date: 08-12-2011
-- Description:	Disables foreign key constraints in the database
-- =============================================
CREATE PROCEDURE [sp_sc_disable_constraints]
AS
DECLARE CONSTRAINTS CURSOR FOR
  SELECT T.TABLE_NAME, T.CONSTRAINT_NAME
  FROM 
    INFORMATION_SCHEMA.CONSTRAINT_TABLE_USAGE T,
    INFORMATION_SCHEMA.REFERENTIAL_CONSTRAINTS RC
  WHERE
    T.CONSTRAINT_CATALOG = RC.CONSTRAINT_CATALOG AND
    T.CONSTRAINT_SCHEMA = RC.CONSTRAINT_SCHEMA AND
    T.CONSTRAINT_NAME = RC.CONSTRAINT_NAME
DECLARE
  @table nvarchar(100),
  @constraint nvarchar(100),
  @sql nvarchar(1000)
BEGIN
  OPEN CONSTRAINTS
  FETCH NEXT FROM CONSTRAINTS INTO @table, @constraint
  WHILE @@FETCH_STATUS <> -1 BEGIN
    SET @sql = 'ALTER TABLE [' + @table + '] NOCHECK CONSTRAINT [' + @constraint + ']';
    EXEC sp_executesql @sql
    FETCH NEXT FROM CONSTRAINTS INTO @table, @constraint
  END
  CLOSE CONSTRAINTS
  DEALLOCATE CONSTRAINTS
END
GO
---------------------------------------------------------------------------------------------
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[sp_sc_enable_constraints]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[sp_sc_enable_constraints]
GO
-- =============================================
-- Author:		Dmitry Kostenko
-- Create date: 08-12-2011
-- Description:	Enables foreign key constraints in the database
-- =============================================
CREATE PROCEDURE [sp_sc_enable_constraints]
AS
DECLARE CONSTRAINTS CURSOR FOR
  SELECT T.TABLE_NAME, T.CONSTRAINT_NAME
  FROM 
    INFORMATION_SCHEMA.CONSTRAINT_TABLE_USAGE T,
    INFORMATION_SCHEMA.REFERENTIAL_CONSTRAINTS RC
  WHERE
    T.CONSTRAINT_CATALOG = RC.CONSTRAINT_CATALOG AND
    T.CONSTRAINT_SCHEMA = RC.CONSTRAINT_SCHEMA AND
    T.CONSTRAINT_NAME = RC.CONSTRAINT_NAME
DECLARE
  @table nvarchar(100),
  @constraint nvarchar(100),
  @sql nvarchar(1000)
BEGIN
  OPEN CONSTRAINTS
  FETCH NEXT FROM CONSTRAINTS INTO @table, @constraint
  WHILE @@FETCH_STATUS <> -1 BEGIN
    SET @sql = 'ALTER TABLE [' + @table + '] CHECK CONSTRAINT [' + @constraint + ']';
    EXEC sp_executesql @sql
    FETCH NEXT FROM CONSTRAINTS INTO @table, @constraint
  END
  CLOSE CONSTRAINTS
  DEALLOCATE CONSTRAINTS
END
GO
---------------------------------------------------------------------------------------------
-- Step 2: Drop obsolete stored procedures
---------------------------------------------------------------------------------------------
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[SET_LOCATION_CLASSIFICATION]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[SET_LOCATION_CLASSIFICATION]
GO
---------------------------------------------------------------------------------------------
-- Step 3: Upgrade the primary tables
---------------------------------------------------------------------------------------------
EXEC sp_sc_disable_indexes 'Visitors'
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_AutomationStates_Visitors]') AND parent_object_id = OBJECT_ID(N'[dbo].[AutomationStates]'))
ALTER TABLE [AutomationStates] DROP CONSTRAINT [FK_AutomationStates_Visitors]
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_Visits_Visitors]') AND parent_object_id = OBJECT_ID(N'[dbo].[Visits]'))
ALTER TABLE [Visits] DROP CONSTRAINT [FK_Visits_Visitors]
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_VisitorTags_Visitors]') AND parent_object_id = OBJECT_ID(N'[dbo].[VisitorTags]'))
ALTER TABLE [VisitorTags]  DROP CONSTRAINT [FK_VisitorTags_Visitors]
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_Profiles_Visitors]') AND parent_object_id = OBJECT_ID(N'[dbo].[Profiles]'))
ALTER TABLE [Profiles]  DROP CONSTRAINT [FK_Profiles_Visitors]
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_Pages_Visitors]') AND parent_object_id = OBJECT_ID(N'[dbo].[Pages]'))
ALTER TABLE [Pages]  DROP CONSTRAINT [FK_Pages_Visitors]
GO
IF  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_PageEvents_Visitors]') AND parent_object_id = OBJECT_ID(N'[dbo].[PageEvents]'))
ALTER TABLE [PageEvents]  DROP CONSTRAINT [FK_PageEvents_Visitors]
GO
IF  EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Visitors]') AND name = N'PK_Visitors')
ALTER TABLE [Visitors] DROP CONSTRAINT [PK_Visitors]
GO
IF not EXISTS (SELECT * FROM INFORMATION_SCHEMA.Columns WHERE TABLE_NAME = N'Visitors' AND COLUMN_NAME = N'rn')
ALTER TABLE [Visitors] ADD [rn] [bigint] IDENTITY
GO
ALTER TABLE [Visitors] alter column ExternalUser nvarchar(256) not null 
GO
IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Visitors]') AND name = N'IX_Order')
CREATE UNIQUE CLUSTERED INDEX [IX_Order] ON [Visitors] ([rn])
WITH (PAD_INDEX  = ON, STATISTICS_NORECOMPUTE  = OFF, SORT_IN_TEMPDB = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON, FILLFACTOR = 100) ON [PRIMARY]
GO
ALTER TABLE [Visitors] ADD  CONSTRAINT [PK_Visitors] PRIMARY KEY NONCLUSTERED
(
	[VisitorId] ASC
)WITH (PAD_INDEX  = ON, STATISTICS_NORECOMPUTE  = OFF, SORT_IN_TEMPDB = OFF, IGNORE_DUP_KEY = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON, FILLFACTOR = 80) ON [PRIMARY]
GO
EXEC sp_sc_enable_indexes 'Visitors'
GO
---------------------------------------------------------------------------------------------
EXEC sp_sc_disable_indexes 'Automations'
GO
IF  EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Automations]') AND name = N'PK_Automation')
ALTER INDEX PK_Automation ON [dbo].[Automations] REBUILD with (PAD_INDEX = ON, FILLFACTOR = 80)
GO
EXEC sp_sc_enable_indexes 'Automations'
GO
---------------------------------------------------------------------------------------------
EXEC sp_sc_disable_indexes 'AutomationStates'
GO
IF  EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[AutomationStates]') AND name = N'PK_AutomationState')
ALTER INDEX PK_AutomationState ON [dbo].[AutomationStates] REBUILD with (PAD_INDEX = ON, FILLFACTOR = 80)
GO
IF  EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[AutomationStates]') AND name = N'IX_State')
DROP INDEX [IX_State] ON [AutomationStates] WITH ( ONLINE = OFF )
GO
IF not EXISTS (SELECT * FROM INFORMATION_SCHEMA.Columns WHERE TABLE_NAME = N'AutomationStates' AND COLUMN_NAME = N'rn')
ALTER TABLE [AutomationStates] ADD [rn] [bigint] IDENTITY
GO
IF  NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[AutomationStates]') AND name = N'IX_Order')
CREATE UNIQUE CLUSTERED INDEX [IX_Order] ON [AutomationStates] ([rn])
WITH (PAD_INDEX  = ON, STATISTICS_NORECOMPUTE  = OFF, SORT_IN_TEMPDB = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON, FILLFACTOR = 100) ON [PRIMARY]
GO
IF  EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[AutomationStates]') AND name = N'IX_ByUser')
DROP INDEX [IX_ByUser] ON [dbo].[AutomationStates] WITH ( ONLINE = OFF )
GO
Alter TABLE [dbo].[AutomationStates]
Alter column [UserName] [nvarchar](256) NULL
GO
CREATE NONCLUSTERED INDEX [IX_ByUser] ON [dbo].[AutomationStates] (	[UserName] ASC)
WHERE ([UserName] IS NOT NULL) WITH (PAD_INDEX  = ON, STATISTICS_NORECOMPUTE  = OFF, SORT_IN_TEMPDB = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON, FILLFACTOR = 80) ON [PRIMARY]
GO
CREATE INDEX [IX_State] ON [AutomationStates] ([State] ASC,[WakeupDateTime] ASC) WITH (PAD_INDEX  = ON, STATISTICS_NORECOMPUTE  = OFF, SORT_IN_TEMPDB = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON, FILLFACTOR = 80) ON [PRIMARY]
GO
alter table AutomationStates NOCHECK constraint FK_MarketingAutomationStates_MarketingAutomations
GO
---------------------------------------------------------------------------------------------
IF  NOT EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_AutomationStates_Visitors]') AND parent_object_id = OBJECT_ID(N'[dbo].[AutomationStates]'))
begin
ALTER TABLE AutomationStates WITH NOCHECK  ADD CONSTRAINT
	FK_AutomationStates_Visitors FOREIGN KEY
	(
	VisitorId
	) REFERENCES Visitors
	(
	VisitorId
	) ON UPDATE  NO ACTION 
	 ON DELETE  NO ACTION
	
end
GO
EXEC sp_sc_enable_indexes 'AutomationStates'
GO
---------------------------------------------------------------------------------------------
EXEC sp_sc_disable_indexes 'Browsers'
GO
IF  EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Browsers]') AND name = N'PK_Browsers')
ALTER INDEX PK_Browsers ON [dbo].[Browsers] REBUILD with (PAD_INDEX = ON, FILLFACTOR = 80)
GO
EXEC sp_sc_enable_indexes 'Browsers'
---------------------------------------------------------------------------------------------
EXEC sp_sc_disable_indexes 'Campaigns'
GO
IF  EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Campaigns]') AND name = N'IX_Category1')
ALTER INDEX [IX_Category1] ON [dbo].[Campaigns] REBUILD with (PAD_INDEX = ON, FILLFACTOR = 80)
GO

IF  EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Campaigns]') AND name = N'IX_Category2')
ALTER INDEX [IX_Category2] ON [dbo].[Campaigns] REBUILD with (PAD_INDEX = ON, FILLFACTOR = 80)
GO

IF  EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Campaigns]') AND name = N'IX_Category3')
ALTER INDEX [IX_Category3] ON [dbo].[Campaigns] REBUILD with (PAD_INDEX = ON, FILLFACTOR = 80)
GO

IF  EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Campaigns]') AND name = N'PK_Campaign')
ALTER INDEX [PK_Campaign] on [dbo].[Campaigns] REBUILD with (PAD_INDEX = ON, FILLFACTOR = 80)
GO
EXEC sp_sc_enable_indexes 'Campaigns'
GO
---------------------------------------------------------------------------------------------
EXEC sp_sc_disable_indexes 'GeoIps'
GO
IF  EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[GeoIps]') AND name = N'PK_GeoIps')
ALTER INDEX [PK_GeoIps] on [dbo].[GeoIps] REBUILD with (PAD_INDEX = ON, FILLFACTOR = 80)
GO
EXEC sp_sc_enable_indexes 'GeoIps'
GO
---------------------------------------------------------------------------------------------
EXEC sp_sc_disable_indexes 'ItemUrls'
GO
IF  EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[ItemUrls]') AND name = N'PK_ItemUrls')
ALTER INDEX PK_ItemUrls ON [dbo].[ItemUrls] REBUILD with (PAD_INDEX = ON, FILLFACTOR = 80)
GO
EXEC sp_sc_enable_indexes 'ItemUrls'
GO
---------------------------------------------------------------------------------------------
EXEC sp_sc_disable_indexes 'Keywords'
GO
IF  EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Keywords]') AND name = N'PK_Keywords')
ALTER INDEX PK_Keywords ON [dbo].[Keywords] REBUILD with (PAD_INDEX = ON, FILLFACTOR = 80)
GO
EXEC sp_sc_enable_indexes 'Keywords'
GO
---------------------------------------------------------------------------------------------
EXEC sp_sc_disable_indexes 'Locations'
GO
IF  EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Locations]') AND name = N'PK_Location')
ALTER INDEX PK_Location ON [dbo].[Locations] REBUILD with (PAD_INDEX = ON, FILLFACTOR = 80)
GO
Alter TABLE [dbo].[Locations]
Alter column [ExternalUser] [nvarchar](256) NOT NULL
GO
alter table [dbo].[Locations] NOCHECK constraint FK_Locations_VisitorClassification
GO
EXEC sp_sc_enable_indexes 'Locations'
GO
---------------------------------------------------------------------------------------------
EXEC sp_sc_disable_indexes 'NotificationSubscriptions'
GO
IF  EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[NotificationSubscriptions]') AND name = N'PK_NotificationSubscriptions')
ALTER INDEX [PK_NotificationSubscriptions] ON [dbo].[NotificationSubscriptions] REBUILD with (PAD_INDEX = ON, FILLFACTOR = 80)
GO
Alter TABLE [dbo].[NotificationSubscriptions]
Alter column [MailRecipients] [nvarchar](254) NOT NULL
GO
alter table [dbo].[NotificationSubscriptions] NOCHECK constraint FK_NotificationSubscriptions_Locations
GO
EXEC sp_sc_enable_indexes 'NotificationSubscriptions'
GO
---------------------------------------------------------------------------------------------
EXEC sp_sc_disable_indexes 'OS'
GO
IF  EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[OS]') AND name = N'PK_OS')
ALTER INDEX [PK_OS] ON [dbo].[OS] REBUILD with (PAD_INDEX = ON, FILLFACTOR = 80)
GO
EXEC sp_sc_enable_indexes 'OS'
GO
---------------------------------------------------------------------------------------------
EXEC sp_sc_disable_indexes 'PageEventDefinitions'
GO
IF not EXISTS (SELECT * FROM INFORMATION_SCHEMA.Columns WHERE TABLE_NAME = N'PageEventDefinitions' AND COLUMN_NAME = N'Category1Label')
ALTER TABLE [PageEventDefinitions] ADD [Category1Label] nvarchar(100) null
GO
IF not EXISTS (SELECT * FROM INFORMATION_SCHEMA.Columns WHERE TABLE_NAME = N'PageEventDefinitions' AND COLUMN_NAME = N'Category2Label')
ALTER TABLE [PageEventDefinitions] ADD [Category2Label] nvarchar(100) null
GO
IF not EXISTS (SELECT * FROM INFORMATION_SCHEMA.Columns WHERE TABLE_NAME = N'PageEventDefinitions' AND COLUMN_NAME = N'Category3Label')
ALTER TABLE [PageEventDefinitions] ADD [Category3Label] nvarchar(100) null
GO
IF not EXISTS (SELECT * FROM INFORMATION_SCHEMA.Columns WHERE TABLE_NAME = N'PageEventDefinitions' AND COLUMN_NAME = N'Category1Id')
ALTER TABLE [PageEventDefinitions] ADD [Category1Id] uniqueidentifier null
GO
IF not EXISTS (SELECT * FROM INFORMATION_SCHEMA.Columns WHERE TABLE_NAME = N'PageEventDefinitions' AND COLUMN_NAME = N'Category2Id')
ALTER TABLE [PageEventDefinitions] ADD [Category2Id] uniqueidentifier null
GO
IF not EXISTS (SELECT * FROM INFORMATION_SCHEMA.Columns WHERE TABLE_NAME = N'PageEventDefinitions' AND COLUMN_NAME = N'Category3Id')
ALTER TABLE [PageEventDefinitions] ADD [Category3Id] uniqueidentifier null
GO
IF  EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[PageEventDefinitions]') AND name = N'PK_PageEventDefinitions')
ALTER INDEX [PK_PageEventDefinitions] ON [dbo].[PageEventDefinitions] REBUILD with (PAD_INDEX = ON, FILLFACTOR = 80)
GO
IF  EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[PageEventDefinitions]') AND name = N'IX_Goals')
ALTER INDEX [IX_Goals] ON [dbo].[PageEventDefinitions] REBUILD with (PAD_INDEX = ON, FILLFACTOR = 80)
GO
EXEC sp_sc_enable_indexes 'PageEventDefinitions'
GO
---------------------------------------------------------------------------------------------
EXEC sp_sc_disable_indexes 'PageEvents'
GO
IF  EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[PageEvents]') AND name = N'IX_ByDate')
DROP INDEX [IX_ByDate] ON [PageEvents] WITH ( ONLINE = OFF )
GO
IF not EXISTS (SELECT * FROM INFORMATION_SCHEMA.Columns WHERE TABLE_NAME = N'PageEvents' AND COLUMN_NAME = N'rn')
ALTER TABLE [PageEvents] ADD [rn] [bigint] IDENTITY
GO
IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[PageEvents]') AND name = N'IX_Order')
CREATE UNIQUE CLUSTERED INDEX [IX_Order] ON [PageEvents] ([rn])
WITH (PAD_INDEX  = ON, STATISTICS_NORECOMPUTE  = OFF, SORT_IN_TEMPDB = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON, FILLFACTOR = 100) ON [PRIMARY]
GO
IF  EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[PageEvents]') AND name = N'PK_PageEvents')
ALTER INDEX [PK_PageEvents] ON [dbo].[PageEvents] REBUILD with (PAD_INDEX = ON, FILLFACTOR = 80)
GO
alter table [dbo].[PageEvents] NOCHECK constraint FK_PageEvents_PageEventDefinitions
GO
alter table [dbo].[PageEvents] NOCHECK constraint FK_PageEvents_Pages
GO
IF not EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_PageEvents_Visitors]') AND parent_object_id = OBJECT_ID(N'[dbo].[PageEvents]'))
begin
ALTER TABLE PageEvents WITH NOCHECK  ADD  CONSTRAINT
	FK_PageEvents_Visitors FOREIGN KEY
	(
	VisitorId
	) REFERENCES Visitors
	(
	VisitorId
	) ON UPDATE  NO ACTION 
	 ON DELETE  NO ACTION 
ALTER TABLE PageEvents NOCHECK  CONSTRAINT FK_PageEvents_Visitors
end	
GO
alter table [dbo].[PageEvents] NOCHECK constraint FK_PageEvents_Visits
GO
IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[PageEvents]') AND name = N'IX_ByDate')
CREATE INDEX [IX_ByDate] ON [PageEvents] 
(
	[DateTime] ASC
)WITH (PAD_INDEX  = ON, STATISTICS_NORECOMPUTE  = OFF, SORT_IN_TEMPDB = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON, FILLFACTOR = 80) ON [PRIMARY]
GO
EXEC sp_sc_enable_indexes 'PageEvents'
GO
---------------------------------------------------------------------------------------------
EXEC sp_sc_disable_indexes 'Pages'
GO
IF  EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Pages]') AND name = N'IX_ByDate')
DROP INDEX [IX_ByDate] ON [Pages] WITH ( ONLINE = OFF )
GO
IF not EXISTS (SELECT * FROM INFORMATION_SCHEMA.Columns WHERE TABLE_NAME = N'Pages' AND COLUMN_NAME = N'rn')
ALTER TABLE [Pages] ADD [rn] [bigint] IDENTITY
GO
IF not EXISTS (SELECT * FROM INFORMATION_SCHEMA.Columns WHERE TABLE_NAME = N'Pages' AND COLUMN_NAME = N'DeviceName')
ALTER TABLE [Pages] ADD [DeviceName] [nvarchar](100) NOT NULL default('')
GO
IF not EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_Pages_Visitors]') AND parent_object_id = OBJECT_ID(N'[dbo].[Pages]'))
begin
ALTER TABLE Pages  WITH NOCHECK ADD CONSTRAINT
	FK_Pages_Visitors FOREIGN KEY
	(
	VisitorId
	) REFERENCES Visitors
	(
	VisitorId
	) ON UPDATE  NO ACTION 
	 ON DELETE  NO ACTION 
end
GO
IF NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Pages]') AND name = N'IX_Order')
CREATE UNIQUE CLUSTERED INDEX [IX_Order] ON [Pages] ([rn])
WITH (PAD_INDEX  = ON, STATISTICS_NORECOMPUTE  = OFF, SORT_IN_TEMPDB = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON, FILLFACTOR = 100) ON [PRIMARY]
GO
CREATE INDEX [IX_ByDate] ON [Pages] 
(
	[DateTime] ASC
)WITH (PAD_INDEX  = ON, STATISTICS_NORECOMPUTE  = OFF, SORT_IN_TEMPDB = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON, FILLFACTOR = 80) ON [PRIMARY]
GO
IF  NOT EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Pages]') AND name = N'IX_ByUrl')
CREATE NONCLUSTERED INDEX [IX_ByUrl] ON [dbo].[Pages] 
(
	[Url] ASC,
	[VisitId] ASC,
	[VisitPageIndex] ASC
)WITH (PAD_INDEX  = ON, STATISTICS_NORECOMPUTE  = OFF, SORT_IN_TEMPDB = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON, FILLFACTOR = 80) ON [PRIMARY]
GO
IF  EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Pages]') AND name = N'PK_Pages')
ALTER INDEX [PK_Pages] ON [dbo].[Pages] REBUILD with (PAD_INDEX = ON, FILLFACTOR = 80)
GO
ALTER TRIGGER [dbo].[UpdateItemUrls] ON [dbo].[Pages] 
FOR INSERT NOT FOR REPLICATION AS
BEGIN
  INSERT INTO ItemUrls (ItemId, Url) 
  SELECT INSERTED.ItemId, MIN(CONVERT(varchar(800), INSERTED.Url)) Url 
  FROM INSERTED LEFT JOIN ItemUrls ON INSERTED.ItemId = ItemUrls.ItemId WHERE ItemUrls.ItemId IS NULL
  GROUP BY INSERTED.ItemId
END
GO
EXEC sp_sc_enable_indexes 'Pages'
GO
---------------------------------------------------------------------------------------------
EXEC sp_sc_disable_indexes 'Profiles'
GO
ALTER TABLE [Profiles] DROP CONSTRAINT [PK_Profile]
GO
IF not EXISTS (SELECT * FROM INFORMATION_SCHEMA.Columns WHERE TABLE_NAME = N'Profiles' AND COLUMN_NAME = N'rn')
ALTER TABLE [Profiles] ADD [rn] [bigint] IDENTITY
GO
IF  not EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Profiles]') AND name = N'IX_Order')
CREATE UNIQUE CLUSTERED INDEX [IX_Order] ON [Profiles] ([rn])
WITH (PAD_INDEX  = ON, STATISTICS_NORECOMPUTE  = OFF, SORT_IN_TEMPDB = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON, FILLFACTOR = 100) ON [PRIMARY]
GO
ALTER TABLE [Profiles] ADD  CONSTRAINT [PK_Profile] PRIMARY KEY NONCLUSTERED
(
	[ProfileId] ASC
)WITH (PAD_INDEX  = ON, STATISTICS_NORECOMPUTE  = OFF, SORT_IN_TEMPDB = OFF, IGNORE_DUP_KEY = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON, FILLFACTOR = 80) ON [PRIMARY]
GO
IF  EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Profiles]') AND name = N'IX_ByVisitId')
ALTER INDEX [IX_ByVisitId] ON [dbo].[Profiles] REBUILD WITH (FILLFACTOR = 80, PAD_INDEX = ON);
GO
IF not  EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_Profiles_Visitors]') AND parent_object_id = OBJECT_ID(N'[dbo].[Profiles]'))
begin
ALTER TABLE Profiles WITH NOCHECK ADD CONSTRAINT
	FK_Profiles_Visitors FOREIGN KEY
	(
	VisitorId
	) REFERENCES Visitors
	(
	VisitorId
	) ON UPDATE  NO ACTION 
	 ON DELETE  NO ACTION  
end
GO
EXEC sp_sc_enable_indexes 'Profiles'
GO
---------------------------------------------------------------------------------------------
EXEC sp_sc_disable_indexes 'ReferringSites'
GO
IF  EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[ReferringSites]') AND name = N'PK_ReferringSites')
ALTER INDEX PK_ReferringSites ON [dbo].[ReferringSites] REBUILD with (PAD_INDEX = ON, FILLFACTOR = 80)
GO
EXEC sp_sc_enable_indexes 'ReferringSites'
GO
---------------------------------------------------------------------------------------------
EXEC sp_sc_disable_indexes 'Screens'
GO
IF  EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Screens]') AND name = N'PK_Screens')
ALTER INDEX PK_Screens ON [dbo].[Screens] REBUILD with (PAD_INDEX = ON, FILLFACTOR = 80)
GO
EXEC sp_sc_enable_indexes 'Screens'
GO
---------------------------------------------------------------------------------------------
EXEC sp_sc_disable_indexes 'Status'
GO
IF  EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Status]') AND name = N'PK_Status')
ALTER INDEX PK_Status ON [dbo].[Status] REBUILD with (PAD_INDEX = ON, FILLFACTOR = 80)
GO
EXEC sp_sc_enable_indexes 'Status'
GO
---------------------------------------------------------------------------------------------
EXEC sp_sc_disable_indexes 'TestDefinitions'
GO
IF  EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[TestDefinitions]') AND name = N'PK_TestDefinitions')
ALTER INDEX PK_TestDefinitions ON [dbo].[TestDefinitions] REBUILD with (PAD_INDEX = ON, FILLFACTOR = 80)
GO
EXEC sp_sc_enable_indexes 'TestDefinitions'
GO
---------------------------------------------------------------------------------------------
EXEC sp_sc_disable_indexes 'TrafficTypes'
GO
IF  EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[TrafficTypes]') AND name = N'PK_TrafficTypes')
ALTER INDEX PK_TrafficTypes ON [dbo].[TrafficTypes] REBUILD with (PAD_INDEX = ON, FILLFACTOR = 80)
GO
EXEC sp_sc_enable_indexes 'TrafficTypes'
GO
---------------------------------------------------------------------------------------------
EXEC sp_sc_disable_indexes 'UserAgents'
GO
IF  EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[UserAgents]') AND name = N'PK_UserAgents')
ALTER INDEX PK_UserAgents ON [dbo].[UserAgents] REBUILD with (PAD_INDEX = ON, FILLFACTOR = 80)
GO
EXEC sp_sc_enable_indexes 'UserAgents'
GO
---------------------------------------------------------------------------------------------
EXEC sp_sc_disable_indexes 'VisitorClassifications'
GO
IF  EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[VisitorClassifications]') AND name = N'PK_VisitorClassification')
ALTER INDEX PK_VisitorClassification ON [dbo].[VisitorClassifications] REBUILD with (PAD_INDEX = ON, FILLFACTOR = 80)
GO
EXEC sp_sc_enable_indexes 'VisitorClassifications'
GO
---------------------------------------------------------------------------------------------
EXEC sp_sc_disable_indexes 'VisitorTags'
GO
IF  EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[VisitorTags]') AND name = N'IX_ByVisitor')
DROP INDEX [IX_ByVisitor] ON [VisitorTags] WITH ( ONLINE = OFF )
GO
IF not EXISTS (SELECT * FROM INFORMATION_SCHEMA.Columns WHERE TABLE_NAME = N'VisitorTags' AND COLUMN_NAME = N'rn')
ALTER TABLE [VisitorTags] ADD [rn] [bigint] IDENTITY
GO
IF not EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[VisitorTags]') AND name = N'IX_Order')
CREATE UNIQUE CLUSTERED INDEX [IX_Order] ON [VisitorTags] ([rn])
WITH (PAD_INDEX  = ON, STATISTICS_NORECOMPUTE  = OFF, SORT_IN_TEMPDB = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON, FILLFACTOR = 100) ON [PRIMARY]
GO

IF  not EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_VisitorTags_Visitors]') AND parent_object_id = OBJECT_ID(N'[dbo].[VisitorTags]'))
begin
ALTER TABLE VisitorTags WITH NOCHECK ADD CONSTRAINT
	FK_VisitorTags_Visitors FOREIGN KEY
	(
	VisitorId
	) REFERENCES Visitors
	(
	VisitorId
	) ON UPDATE  NO ACTION 
	 ON DELETE  NO ACTION 
end
GO
IF not EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[VisitorTags]') AND name = N'IX_ByVisitor')
CREATE INDEX [IX_ByVisitor] ON [VisitorTags] 
(
	[VisitorId] ASC
)WITH (PAD_INDEX  = ON, STATISTICS_NORECOMPUTE  = OFF, SORT_IN_TEMPDB = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON, FILLFACTOR = 80) ON [PRIMARY]
GO
IF  EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[VisitorTags]') AND name = N'PK_VisitorTags')
ALTER INDEX PK_VisitorTags ON [dbo].[VisitorTags] REBUILD with (PAD_INDEX = ON, FILLFACTOR = 80)
GO
EXEC sp_sc_enable_indexes 'VisitorTags'
GO
---------------------------------------------------------------------------------------------
EXEC sp_sc_disable_indexes 'Visits'
GO
IF EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Visits]') AND name = N'IX_ByDate')
DROP INDEX [IX_ByDate] ON [Visits] WITH ( ONLINE = OFF )
GO

IF not EXISTS (SELECT * FROM INFORMATION_SCHEMA.Columns WHERE TABLE_NAME = N'Visits' AND COLUMN_NAME = N'rn')
ALTER TABLE [Visits] ADD [rn] [bigint] IDENTITY
GO
IF not EXISTS (SELECT * FROM INFORMATION_SCHEMA.Columns WHERE TABLE_NAME = N'Visits' AND COLUMN_NAME = N'State')
ALTER TABLE [Visits] ADD [State] [tinyint] NOT NULL default(0)
GO
IF not EXISTS (SELECT * FROM INFORMATION_SCHEMA.Columns WHERE TABLE_NAME = N'Visits' AND COLUMN_NAME = N'StateChanged')
ALTER TABLE [Visits] ADD [StateChanged] [smalldatetime] NULL
GO
IF not EXISTS (SELECT * FROM INFORMATION_SCHEMA.Columns WHERE TABLE_NAME = N'Visits' AND COLUMN_NAME = N'DeviceName')
ALTER TABLE [Visits] ADD [DeviceName] [nvarchar](100) NOT NULL default('')
GO
IF not EXISTS (SELECT * FROM INFORMATION_SCHEMA.Columns WHERE TABLE_NAME = N'Visits' AND COLUMN_NAME = N'Language')
ALTER TABLE [Visits] ADD [Language] [nvarchar](50) NOT NULL default('')
GO

IF not EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Visits]') AND name = N'IX_Order')
CREATE UNIQUE CLUSTERED INDEX [IX_Order] ON [Visits] ([rn])
WITH (PAD_INDEX  = ON, STATISTICS_NORECOMPUTE  = OFF, SORT_IN_TEMPDB = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON, FILLFACTOR = 100) ON [PRIMARY]
GO
IF  not EXISTS (SELECT * FROM sys.foreign_keys WHERE object_id = OBJECT_ID(N'[dbo].[FK_Visits_Visitors]') AND parent_object_id = OBJECT_ID(N'[dbo].[Visits]'))
begin
ALTER TABLE Visits WITH NOCHECK ADD CONSTRAINT
	FK_Visits_Visitors FOREIGN KEY
	(
	VisitorId
	) REFERENCES Visitors
	(
	VisitorId
	) ON UPDATE  NO ACTION 
	 ON DELETE  NO ACTION
end	
GO
IF not EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Visits]') AND name = N'IX_ByDate')
CREATE NONCLUSTERED INDEX [IX_ByDate] ON [dbo].[Visits] 
(
	[StartDateTime] ASC,
	[VisitorId] ASC,
	[VisitId] ASC
)WITH (PAD_INDEX  = ON, STATISTICS_NORECOMPUTE  = OFF, SORT_IN_TEMPDB = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON, FILLFACTOR = 80) ON [PRIMARY]
GO
IF not EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Visits]') AND name = N'IX_ByState')
CREATE NONCLUSTERED INDEX [IX_ByState] ON [dbo].[Visits] 
(
	[State] ASC
)WITH (PAD_INDEX  = ON, STATISTICS_NORECOMPUTE  = OFF, SORT_IN_TEMPDB = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON, FILLFACTOR = 80) ON [PRIMARY]
GO
IF  EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Visits]') AND name = N'IX_ByLocation')
ALTER INDEX IX_ByLocation ON [dbo].[Visits] REBUILD with (PAD_INDEX = ON, FILLFACTOR = 80)
GO
IF  EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Visits]') AND name = N'IX_ByLocation')
ALTER INDEX IX_ByLocation ON [dbo].[Visits] REBUILD with (PAD_INDEX = ON, FILLFACTOR = 80)
GO
IF  EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Visits]') AND name = N'IX_ByReferringSite')
ALTER INDEX IX_ByReferringSite ON [dbo].[Visits] REBUILD with (PAD_INDEX = ON, FILLFACTOR = 80)
GO
IF  EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Visits]') AND name = N'IX_ByReferringSite')
ALTER INDEX PK_Visits ON [dbo].[Visits] REBUILD with (PAD_INDEX = ON, FILLFACTOR = 80)

EXEC sp_sc_enable_indexes 'Visits'
GO
---------------------------------------------------------------------------------------------
-- Step 4: Upgrade the reporting views
---------------------------------------------------------------------------------------------
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[Cache_TrafficByDay]') AND type in (N'U'))
DROP TABLE [dbo].[Cache_TrafficByDay]
GO
CREATE TABLE [dbo].[Cache_TrafficByDay](
	[Date] [char](8) NULL,
	[Month] [char](8) NULL,
	[TrafficType] [int] NOT NULL,
	[CampaignId] [uniqueidentifier] NULL,
	[ItemId] [uniqueidentifier] NOT NULL,
	[Url] [varchar](800) NULL,
	[KeywordsId] [uniqueidentifier] NOT NULL,
	[Keywords] [nvarchar](400) NULL,
	[ReferringSiteId] [uniqueidentifier] NOT NULL,
	[ReferringSite] [varchar](100) NULL,
	[Multisite] [nvarchar](450) NULL,
	[DeviceName] [nvarchar](100) NULL,
	[Language] [nvarchar](50) NULL,
	[FirstVisit] [int] NOT NULL,
	[Visits] [int] NULL,
	[Value] [int] NULL,
	[rn] [bigint] IDENTITY(1,1) NOT NULL,
 CONSTRAINT [Cache_TrafficByDay_PKey] PRIMARY KEY CLUSTERED 
(
	[rn] ASC
)WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]
GO

IF  EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Cache_TrafficByDay]') AND name = N'IX_ByCampaign')
DROP INDEX [IX_ByCampaign] ON [dbo].[Cache_TrafficByDay] WITH ( ONLINE = OFF )
GO
CREATE NONCLUSTERED INDEX [IX_ByCampaign] ON [dbo].[Cache_TrafficByDay] 
(
	[CampaignId] ASC,
	[Month] ASC,
	[Date] ASC
)
INCLUDE ( [Keywords],
[ReferringSite],
[ItemId],
[Url],
[Visits],
[Value]) WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, SORT_IN_TEMPDB = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON, FILLFACTOR = 90) ON [PRIMARY]
GO

IF  EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Cache_TrafficByDay]') AND name = N'IX_ByDate')
DROP INDEX [IX_ByDate] ON [dbo].[Cache_TrafficByDay] WITH ( ONLINE = OFF )
GO

CREATE NONCLUSTERED INDEX [IX_ByDate] ON [dbo].[Cache_TrafficByDay] 
(
	[Date] ASC,
	[TrafficType] ASC,
	[KeywordsId] ASC,
	[ReferringSiteId] ASC,
	[CampaignId] ASC,
	[ItemId] ASC,
	[FirstVisit] ASC
)
INCLUDE ( [Month],
[Keywords],
[ReferringSite],
[Url],
[Visits],
[Value]) WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, SORT_IN_TEMPDB = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON, FILLFACTOR = 90) ON [PRIMARY]
GO

IF  EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Cache_TrafficByDay]') AND name = N'IX_ByType')
DROP INDEX [IX_ByType] ON [dbo].[Cache_TrafficByDay] WITH ( ONLINE = OFF )

GO
CREATE NONCLUSTERED INDEX [IX_ByType] ON [dbo].[Cache_TrafficByDay] 
(
	[TrafficType] ASC,
	[Month] ASC,
	[Date] ASC
)
INCLUDE ( [Keywords],
[ReferringSite],
[CampaignId],
[ItemId],
[Url],
[Visits],
[Value]) WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, SORT_IN_TEMPDB = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON, FILLFACTOR = 90) ON [PRIMARY]
GO

---------------------------------------------------------------------------------------------
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[Cache_ValueBySource]') AND type in (N'U'))
DROP TABLE [dbo].[Cache_ValueBySource]
GO
CREATE TABLE [dbo].[Cache_ValueBySource](
	[Date] [char](8) NULL,
	[TrafficType] [int] NOT NULL,
	[Multisite] [nvarchar](450) NULL,
	[DeviceName] [nvarchar](100) NULL,
	[Language] [nvarchar](50) NULL,
	[Visits] [int] NULL,
	[Value] [int] NULL,
	[FirstVisitValue] [int] NULL,
	[Visitors] [bigint] NULL,
	[rn] [bigint] IDENTITY(1,1) NOT NULL,
 CONSTRAINT [Cache_ValueBySource_PKey] PRIMARY KEY CLUSTERED 
(
	[rn] ASC
)WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]
GO

IF  EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Cache_ValueBySource]') AND name = N'IX_ByDate')
DROP INDEX [IX_ByDate] ON [dbo].[Cache_ValueBySource] WITH ( ONLINE = OFF )
GO

CREATE NONCLUSTERED INDEX [IX_ByDate] ON [dbo].[Cache_ValueBySource] 
(
	[Date] ASC,
	[TrafficType] ASC
)
INCLUDE ( [Visits],
[Value],
[FirstVisitValue],
[Visitors]) WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, SORT_IN_TEMPDB = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON, FILLFACTOR = 90) ON [PRIMARY]
GO
---------------------------------------------------------------------------------------------
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[Cache_VisitEvents]') AND type in (N'U'))
DROP TABLE [dbo].[Cache_VisitEvents]
GO
CREATE TABLE [dbo].[Cache_VisitEvents](
	[Date] [char](8) NULL,
	[Month] [char](8) NULL,
	[VisitId] [uniqueidentifier] NOT NULL,
	[TrafficType] [int] NOT NULL,
	[CampaignId] [uniqueidentifier] NULL,
	[Multisite] [nvarchar](450) NULL,
	[DeviceName] [nvarchar](100) NULL,
	[Language] [nvarchar](50) NULL,
	[Value] [int] NOT NULL,
	[PageEventDefinitionId] [uniqueidentifier] NOT NULL,
	[PageEventName] [nvarchar](100) NOT NULL,
	[IsGoal] [bit] NOT NULL,
	[ItemId] [uniqueidentifier] NOT NULL,
	[EventItemId] [uniqueidentifier] NOT NULL,
	[DataKey] [varchar](100) NULL,
	[NumberOfEvents] [int] NULL,
	[rn] [bigint] IDENTITY(1,1) NOT NULL,
 CONSTRAINT [Cache_VisitEvents_PKey] PRIMARY KEY CLUSTERED 
(
	[rn] ASC
)WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]
GO

IF  EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Cache_VisitEvents]') AND name = N'IX_ByCampaign')
DROP INDEX [IX_ByCampaign] ON [dbo].[Cache_VisitEvents] WITH ( ONLINE = OFF )
GO

CREATE NONCLUSTERED INDEX [IX_ByCampaign] ON [dbo].[Cache_VisitEvents] 
(
	[IsGoal] ASC,
	[CampaignId] ASC,
	[PageEventDefinitionId] ASC,
	[Month] ASC,
	[Date] ASC,
	[DataKey] ASC
)
INCLUDE ( [Value],
[NumberOfEvents],
[TrafficType],
[VisitId],
[PageEventName]) WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, SORT_IN_TEMPDB = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON, FILLFACTOR = 90) ON [PRIMARY]
GO

IF  EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Cache_VisitEvents]') AND name = N'IX_ByDefinition')
DROP INDEX [IX_ByDefinition] ON [dbo].[Cache_VisitEvents] WITH ( ONLINE = OFF )
GO

CREATE NONCLUSTERED INDEX [IX_ByDefinition] ON [dbo].[Cache_VisitEvents] 
(
	[IsGoal] ASC,
	[PageEventDefinitionId] ASC,
	[CampaignId] ASC,
	[Month] ASC,
	[Date] ASC,
	[DataKey] ASC
)
INCLUDE ( [Value],
[NumberOfEvents],
[TrafficType],
[VisitId],
[PageEventName]) WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, SORT_IN_TEMPDB = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON, FILLFACTOR = 90) ON [PRIMARY]
GO

IF  EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Cache_VisitEvents]') AND name = N'IX_ByItemId')
DROP INDEX [IX_ByItemId] ON [dbo].[Cache_VisitEvents] WITH ( ONLINE = OFF )
GO

CREATE NONCLUSTERED INDEX [IX_ByItemId] ON [dbo].[Cache_VisitEvents] 
(
	[IsGoal] ASC,
	[ItemId] ASC,
	[Date] ASC,
	[PageEventDefinitionId] ASC,
	[DataKey] ASC
)
INCLUDE ( [Value],
[NumberOfEvents],
[Month],
[PageEventName],
[CampaignId],
[VisitId]) WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, SORT_IN_TEMPDB = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON, FILLFACTOR = 90) ON [PRIMARY]
GO

IF  EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Cache_VisitEvents]') AND name = N'IX_ByVisit')
DROP INDEX [IX_ByVisit] ON [dbo].[Cache_VisitEvents] WITH ( ONLINE = OFF )
GO

CREATE NONCLUSTERED INDEX [IX_ByVisit] ON [dbo].[Cache_VisitEvents] 
(
	[IsGoal] ASC,
	[VisitId] ASC,
	[PageEventDefinitionId] ASC,
	[DataKey] ASC,
	[ItemId] ASC,
	[EventItemId] ASC
)
INCLUDE ( [Date],
[Month],
[TrafficType],
[CampaignId],
[Value],
[PageEventName],
[NumberOfEvents]) WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, SORT_IN_TEMPDB = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON, FILLFACTOR = 90) ON [PRIMARY]
GO

---------------------------------------------------------------------------------------------
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[Cache_VisitorsByLocation]') AND type in (N'U'))
DROP TABLE [dbo].[Cache_VisitorsByLocation]
GO
CREATE TABLE [dbo].[Cache_VisitorsByLocation](
	[Date] [char](8) NULL,
	[VisitorClassification] [int] NOT NULL,
	[LocationId] [uniqueidentifier] NOT NULL,
	[Country] [nvarchar](100) NOT NULL,
	[Region] [nvarchar](100) NOT NULL,
	[VisitorId] [uniqueidentifier] NOT NULL,
	[Visits] [bigint] NULL,
	[Value] [int] NULL,
	[rn] [bigint] IDENTITY(1,1) NOT NULL,
 CONSTRAINT [Cache_VisitorsByLocation_PKey] PRIMARY KEY CLUSTERED 
(
	[rn] ASC
)WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON) ON [PRIMARY]
) ON [PRIMARY]
GO

IF  EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[dbo].[Cache_VisitorsByLocation]') AND name = N'IX_VisitorsByLocation')
DROP INDEX [IX_VisitorsByLocation] ON [dbo].[Cache_VisitorsByLocation] WITH ( ONLINE = OFF )
GO

CREATE NONCLUSTERED INDEX [IX_VisitorsByLocation] ON [dbo].[Cache_VisitorsByLocation] 
(
	[Date] ASC,
	[VisitorClassification] ASC,
	[Country] ASC,
	[Region] ASC,
	[LocationId] ASC,
	[VisitorId] ASC
)
INCLUDE ( [Visits],
[Value]) WITH (PAD_INDEX  = OFF, STATISTICS_NORECOMPUTE  = OFF, SORT_IN_TEMPDB = OFF, IGNORE_DUP_KEY = OFF, DROP_EXISTING = OFF, ONLINE = OFF, ALLOW_ROW_LOCKS  = ON, ALLOW_PAGE_LOCKS  = ON, FILLFACTOR = 90) ON [PRIMARY]
GO

---------------------------------------------------------------------------------------------
IF  EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'[dbo].[TrafficByDay]'))
DROP VIEW [dbo].[TrafficByDay]
GO
CREATE VIEW [dbo].[TrafficByDay]
AS SELECT * FROM [Cache_TrafficByDay]
GO
---------------------------------------------------------------------------------------------
IF  EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'[dbo].[TrafficOverview]'))
DROP VIEW [dbo].[TrafficOverview]
GO

CREATE VIEW [dbo].[TrafficOverview] AS
SELECT [Date]
      ,[Month]
      ,[TrafficType]
      ,[CampaignId]
      ,[ItemId]
      ,[Url]
      ,[KeywordsId]
      ,[Keywords]
      ,[ReferringSiteId]
      ,[ReferringSite]
      ,[Multisite]
      ,[Language]
      ,[DeviceName]
      ,[FirstVisit]
      ,[Visits]
      ,[Value]
  FROM [TrafficByDay]

GO
---------------------------------------------------------------------------------------------
IF  EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'[dbo].[CampaignsOverview]'))
DROP VIEW [dbo].[CampaignsOverview]
GO
CREATE VIEW [dbo].[CampaignsOverview]
AS
SELECT 
  [TrafficOverview].[Date],
  [TrafficOverview].[Month],
  [TrafficOverview].[ItemId],
  [TrafficOverview].[Url], 
  [TrafficOverview].[KeywordsId],
  [TrafficOverview].[Keywords],
  [TrafficOverview].[ReferringSiteId],
  [TrafficOverview].[ReferringSite],
  [TrafficOverview].[Multisite],
  [TrafficOverview].[Language],
  [TrafficOverview].[DeviceName],
  [TrafficOverview].[FirstVisit],
  [Campaigns].[CampaignName],
  [Campaigns].[Category1Id],
  [Campaigns].[Category2Id],
  [Campaigns].[Category3Id],
  [Campaigns].[CampaignId],
  [TrafficOverview].[Visits],
  [TrafficOverview].[Value]
FROM [TrafficOverview]
  INNER JOIN [Campaigns] ON [TrafficOverview].[CampaignId] = [Campaigns].[CampaignId]
GO
---------------------------------------------------------------------------------------------
IF  EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'[dbo].[Source_TrafficByDay]'))
DROP VIEW [dbo].[Source_TrafficByDay]
GO
CREATE VIEW [dbo].[Source_TrafficByDay] AS 
SELECT
  CONVERT(char(8), [Visits].[StartDateTime], 112) [Date], 
  CAST(CONVERT(char(6), [Visits].[StartDateTime], 112) + '01' AS char(8)) [Month],
  [Visits].[TrafficType], 
  [Visits].[CampaignId], 
  [Pages].[ItemId], 
  LOWER(SUBSTRING(CONVERT(varchar(800), [ItemUrls].[Url]), 1, CHARINDEX('?', [ItemUrls].[Url] + '?') - 1)) [Url], 
  [Visits].[KeywordsId], 
  LOWER(CONVERT(nvarchar(400), [Keywords].[Text])) [Keywords], 
  [Visits].[ReferringSiteId], 
  LOWER([ReferringSites].[Host]) [ReferringSite],
  LOWER([Visits].[Multisite]) [Multisite],
  LOWER([Visits].[DeviceName]) [DeviceName],
  LOWER([Visits].[Language]) [Language],
  (CASE WHEN [Visits].[VisitorVisitIndex] = 1 THEN 1 ELSE 0 END) [FirstVisit],
  COUNT(*) AS [Visits], 
  SUM([Visits].[Value]) AS [Value]
FROM 
  [Visits] WITH (NOLOCK)
  INNER JOIN [Keywords] WITH (NOLOCK) ON [Visits].[KeywordsId] = [Keywords].[KeywordsId] 
  INNER JOIN [Pages] WITH (NOLOCK) ON [Visits].[VisitId] = [Pages].[VisitId] AND [Pages].[VisitPageIndex] = 1 AND [Pages].[ItemId] <> '00000000-0000-0000-0000-000000000000' 
  INNER JOIN [ItemUrls] WITH (NOLOCK) ON [Pages].[ItemId] = [ItemUrls].[ItemId] 
  INNER JOIN [ReferringSites] WITH (NOLOCK) ON [Visits].[ReferringSiteId] = [ReferringSites].[ReferringSiteId]
GROUP BY 
  CONVERT(char(8), [Visits].[StartDateTime], 112),
  CONVERT(char(6), [Visits].[StartDateTime], 112) + '01',
  [Visits].[TrafficType],
  [Visits].[CampaignId],
  [Pages].[ItemId],
  LOWER(SUBSTRING(CONVERT(varchar(800), [ItemUrls].[Url]), 1, CHARINDEX('?', [ItemUrls].[Url] + '?') - 1)), 
  [Visits].[KeywordsId],
  LOWER(CONVERT(nvarchar(400), [Keywords].[Text])), 
  [Visits].[ReferringSiteId],
  LOWER([ReferringSites].[Host]),
  LOWER([Visits].[MultiSite]),
  LOWER([Visits].[DeviceName]),
  LOWER([Visits].[Language]),
  (CASE WHEN [Visits].[VisitorVisitIndex] = 1 THEN 1 ELSE 0 END)

GO
---------------------------------------------------------------------------------------------
IF  EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'[dbo].[Source_ValueBySource]'))
DROP VIEW [dbo].[Source_ValueBySource]
GO
CREATE VIEW [dbo].[Source_ValueBySource] AS
SELECT 
  CONVERT(char(8), [Visits].[StartDateTime], 112) [Date], 
  [Visits].[TrafficType], 
  LOWER([Visits].[Multisite]) [Multisite], 
  LOWER([Visits].[DeviceName]) [DeviceName], 
  LOWER([Visits].[Language]) [Language],
  SUM([Visitors].[VisitCount]) [Visits], 
  SUM([Visitors].[Value]) [Value], 
  SUM([Visits].[Value]) [FirstVisitValue],
  COUNT_BIG(*) [Visitors]
FROM 
  [Visitors] WITH (NOLOCK)
  INNER JOIN [Visits] WITH (NOLOCK) ON [Visits].[VisitorVisitIndex] = 1 AND [Visits].[VisitorId] = [Visitors].[VisitorId]
GROUP BY 
  CONVERT(char(8), [Visits].[StartDateTime], 112), 
  [TrafficType],
  LOWER([Visits].[Multisite]), 
  LOWER([Visits].[DeviceName]), 
  LOWER([Visits].[Language])

GO
---------------------------------------------------------------------------------------------
IF  EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'[dbo].[Source_VisitEvents]'))
DROP VIEW [dbo].[Source_VisitEvents]
GO
CREATE VIEW [dbo].[Source_VisitEvents] AS
SELECT
  CONVERT(char(8), [Visits].[StartDateTime], 112) AS [Date],
  CAST(CONVERT(char(6), [Visits].[StartDateTime], 112) + '01' AS char(8)) AS [Month],
  [Visits].[VisitId],
  [Visits].[TrafficType],
  [Visits].[CampaignId],
  LOWER([Visits].[Multisite]) [Multisite],
  LOWER([Visits].[DeviceName]) [DeviceName],
  LOWER([Visits].[Language]) [Language],
  [Visits].[Value],
  [PageEvents].[PageEventDefinitionId],
  [PageEventDefinitions].[Name] [PageEventName], 
  [PageEventDefinitions].[IsGoal],
  [Pages].[ItemId],
  [PageEvents].[ItemId] [EventItemId],
  LOWER([PageEvents].[DataKey]) [DataKey],
  COUNT(*) [NumberOfEvents]
FROM 
  [PageEvents] WITH (NOLOCK)
  INNER JOIN [Visits] WITH (NOLOCK) ON [Visits].[VisitId] = [PageEvents].[VisitId]
  INNER JOIN [PageEventDefinitions] WITH (NOLOCK) ON [PageEvents].[PageEventDefinitionId] = [PageEventDefinitions].[PageEventDefinitionId]
  INNER JOIN [Pages] WITH (NOLOCK) ON [PageEvents].[PageId] = [Pages].[PageId] AND [PageEvents].[VisitId] = [Pages].[VisitId]
WHERE 
  [PageEvents].[DataKey] > ' ' OR [PageEventDefinitions].[IsGoal] = 1
GROUP BY 
  CONVERT(char(8), [Visits].[StartDateTime], 112),
  CONVERT(char(6), [Visits].[StartDateTime], 112) + '01',
  [Visits].[VisitId],
  [Visits].[TrafficType],
  [Visits].[CampaignId],
  LOWER([Visits].[Multisite]),
  LOWER([Visits].[DeviceName]),
  LOWER([Visits].[Language]),
  [Visits].[Value], 
  [PageEvents].[PageEventDefinitionId],
  [PageEventDefinitions].[Name],
  [PageEventDefinitions].[IsGoal],
  LOWER([PageEvents].[DataKey]),
  [Pages].[ItemId],
  [PageEvents].[ItemId]

GO
---------------------------------------------------------------------------------------------
IF  EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'[dbo].[Source_VisitorsByLocation]'))
DROP VIEW [dbo].[Source_VisitorsByLocation]
GO

CREATE VIEW [dbo].[Source_VisitorsByLocation] AS
SELECT
    CONVERT(char(8), [StartDateTime], 112) [Date],
    [Visitors].[VisitorClassification],
	  [LocationId],
	  [Country],
	  [Region],
	  [Visits].[VisitorId],
	  COUNT_BIG(*) [Visits],
	  SUM([Visits].[Value]) [Value]
FROM
	  [Visits] WITH (NOLOCK), [Visitors] WITH (NOLOCK)
WHERE	
	  [Visits].[VisitorId] = [Visitors].[VisitorId] AND
	  [Visitors].[VisitorClassification] < 900
GROUP BY
	  [Visits].[VisitorId],
		[LocationId],
		[Country],
		[Region],
		[VisitorClassification],
		CONVERT(char(8), [StartDateTime], 112)

GO
---------------------------------------------------------------------------------------------
IF  EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'[dbo].[ValueBySource]'))
DROP VIEW [dbo].[ValueBySource]
GO
CREATE VIEW [dbo].[ValueBySource]
AS SELECT * FROM [Cache_ValueBySource]
GO
---------------------------------------------------------------------------------------------
IF  EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'[dbo].[VisitEvents]'))
DROP VIEW [dbo].[VisitEvents]
GO
CREATE VIEW [dbo].[VisitEvents]
AS SELECT * FROM [Cache_VisitEvents]
GO
---------------------------------------------------------------------------------------------
IF  EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'[dbo].[VisitEventsByDataKey]'))
DROP VIEW [dbo].[VisitEventsByDataKey]
GO
CREATE VIEW [dbo].[VisitEventsByDataKey]
AS
SELECT 
  [Date], 
  [Month],
  [TrafficType],
  [CampaignId],
  [VisitId],
  [PageEventDefinitionId],
  [PageEventName],
  [DataKey],
  [Multisite],
  [Language],
  [DeviceName],
  [Value],
  [IsGoal],
  SUM([NumberOfEvents]) [NumberOfEvents]
FROM [VisitEvents]
GROUP BY
  [Date], 
  [Month],
  [TrafficType],
  [CampaignId],
  [VisitId],
  [PageEventDefinitionId],
  [PageEventName],
  [DataKey],
  [Multisite],
  [Language],
  [DeviceName],
  [Value],
  [IsGoal]
GO
---------------------------------------------------------------------------------------------
IF  EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'[dbo].[VisitEventsByDefinition]'))
DROP VIEW [dbo].[VisitEventsByDefinition]
GO
CREATE VIEW [dbo].[VisitEventsByDefinition]
AS
SELECT 
  [Date], 
  [Month],
  [TrafficType],
  [CampaignId],
  [VisitId],
  [PageEventDefinitionId],
  [PageEventName],
  [Multisite],
  [Language],
  [DeviceName],
  [Value],
  [IsGoal],
  SUM([NumberOfEvents]) [NumberOfEvents]
FROM [VisitEvents]
GROUP BY
  [Date], 
  [Month],
  [TrafficType],
  [CampaignId],
  [VisitId],
  [PageEventDefinitionId],
  [PageEventName],
  [Multisite],
  [Language],
  [DeviceName],
  [Value],
  [IsGoal]
GO
---------------------------------------------------------------------------------------------
IF  EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'[dbo].[VisitEventsByPage]'))
DROP VIEW [dbo].[VisitEventsByPage]
GO
CREATE VIEW [dbo].[VisitEventsByPage]
AS
SELECT 
  [Date], 
  [Month],
  [TrafficType],
  [CampaignId],
  [VisitId],
  [PageEventDefinitionId],
  [PageEventName],
  [ItemId],
  [Multisite],
  [Language],
  [DeviceName],
  [Value],
  [IsGoal],
  SUM([NumberOfEvents]) [NumberOfEvents]
FROM [VisitEvents]
GROUP BY
  [ItemId],
  [PageEventName],
  [IsGoal],
  [TrafficType],
  [CampaignId],
  [Date], 
  [Month],
  [VisitId],
  [PageEventDefinitionId],
  [Multisite],
  [Language],
  [DeviceName],
  [Value]
GO
---------------------------------------------------------------------------------------------
IF  EXISTS (SELECT * FROM sys.views WHERE object_id = OBJECT_ID(N'[dbo].[VisitorsByLocation]'))
DROP VIEW [dbo].[VisitorsByLocation]
GO
CREATE VIEW [dbo].[VisitorsByLocation]
AS SELECT * FROM [Cache_VisitorsByLocation]
GO
---------------------------------------------------------------------------------------------
-- Stored procedure to update the view data
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[sp_sc_update_TrafficByDay]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[sp_sc_update_TrafficByDay]
GO
CREATE PROCEDURE [sp_sc_update_TrafficByDay] AS
BEGIN
  EXEC sp_sc_disable_indexes 'Cache_TrafficByDay';

  TRUNCATE TABLE [Cache_TrafficByDay];

  INSERT INTO [Cache_TrafficByDay] WITH (TABLOCK)
  SELECT * FROM [Source_TrafficByDay];
             
  EXEC sp_sc_enable_indexes 'Cache_TrafficByDay';
END
GO
---------------------------------------------------------------------------------------------
-- Stored procedure to update the view data
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[sp_sc_update_ValueBySource]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[sp_sc_update_ValueBySource]
GO
CREATE PROCEDURE [sp_sc_update_ValueBySource] AS
BEGIN
  EXEC sp_sc_disable_indexes 'Cache_ValueBySource';

  TRUNCATE TABLE [Cache_ValueBySource];

  INSERT INTO [Cache_ValueBySource] WITH (TABLOCK)
  SELECT * FROM [Source_ValueBySource];
             
  EXEC sp_sc_enable_indexes 'Cache_ValueBySource';
END
GO
---------------------------------------------------------------------------------------------
-- Stored procedure to update the view data
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[sp_sc_update_VisitEvents]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[sp_sc_update_VisitEvents]
GO
CREATE PROCEDURE [sp_sc_update_VisitEvents] AS
BEGIN
  EXEC sp_sc_disable_indexes 'Cache_VisitEvents';

  TRUNCATE TABLE [Cache_VisitEvents];

  INSERT INTO [Cache_VisitEvents] WITH (TABLOCK)
  SELECT * FROM [Source_VisitEvents];
             
  EXEC sp_sc_enable_indexes 'Cache_VisitEvents';
END
GO
---------------------------------------------------------------------------------------------
-- Stored procedure to update the view data
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[sp_sc_update_VisitorsByLocation]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[sp_sc_update_VisitorsByLocation]
GO
CREATE PROCEDURE [sp_sc_update_VisitorsByLocation] AS
BEGIN
  EXEC sp_sc_disable_indexes 'Cache_VisitorsByLocation';

  TRUNCATE TABLE [Cache_VisitorsByLocation];

  INSERT INTO [Cache_VisitorsByLocation] WITH (TABLOCK)
  SELECT * FROM [Source_VisitorsByLocation];
             
  EXEC sp_sc_enable_indexes 'Cache_VisitorsByLocation';
END
GO
---------------------------------------------------------------------------------------------
-- Stored procedure to refresh the view data
IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'[dbo].[sp_sc_refresh_views]') AND type in (N'P', N'PC'))
DROP PROCEDURE [dbo].[sp_sc_refresh_views]
GO
CREATE PROCEDURE [sp_sc_refresh_views] AS
BEGIN
  EXEC sp_sc_update_TrafficByDay
  EXEC sp_sc_update_VisitEvents
  EXEC sp_sc_update_ValueBySource
  EXEC sp_sc_update_VisitorsByLocation
END
GO
---------------------------------------------------------------------------------------------
-- Step 5: Prepare database for use
---------------------------------------------------------------------------------------------
EXEC sp_sc_disable_constraints
GO
